"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CoronerdEvent = void 0;
const fs_1 = __importDefault(require("fs"));
const fs_extra_1 = require("fs-extra");
const util_1 = __importDefault(require("util"));
const symbolGroup_1 = require("./symbolGroup");
class CoronerdEvent {
    constructor(eventPath) {
        this.eventPath = eventPath;
        this._eventLoaded = false;
        this._loaded = false;
        this._winExtensions = ['.dll', '.exe'];
    }
    async eventExists() {
        if (this._eventLoaded === true) {
            return true;
        }
        return await util_1.default.promisify(fs_1.default.exists)(this.eventPath);
    }
    async loadEvent() {
        if (this._loaded) {
            return true;
        }
        if (!this._eventLoaded) {
            await this.readEvent();
            if (!this._eventLoaded) {
                return false;
            }
        }
        if (this.validateEvent()) {
            throw new Error(`Coronerd generated message with empty missing_symbols property. Object won't be available in symbold`);
        }
        this.generateMissingSymbolsData();
        this._loaded = true;
        this.numberOfConvertedSymbols = 0;
        return this._loaded;
    }
    validateEvent() {
        return !this.missing_symbols || this.missing_symbols.length === 0;
    }
    async readEvent() {
        const file = await fs_extra_1.readFile(this.eventPath, 'utf-8');
        if (!file) {
            return;
        }
        const plainObject = JSON.parse(file);
        plainObject._eventLoaded = true;
        Object.assign(this, plainObject);
    }
    generateMissingSymbolsData() {
        const isWindows = this.os.toLowerCase().startsWith('win');
        const groupData = symbolGroup_1.SymbolGroup.fromMissing(isWindows, this.missing_symbols);
        this.missingSymbolsData = {};
        for (const groupBase in groupData) {
            const group = groupData[groupBase];
            for (const ident of group.identifiers) {
                for (const basename of group.basenames) {
                    for (const extension of group.extensions) {
                        let fileExtension = extension;
                        if (isWindows && this._winExtensions.indexOf(extension) >= 0) {
                            fileExtension = '.pdb';
                        }
                        const fileName = `${basename}${fileExtension}`;
                        const missingSymbol = `${basename}${extension},${ident}`;
                        if (!this.missingSymbolsData[fileName]) {
                            this.missingSymbolsData[fileName] = new Set();
                        }
                        if (this.missing_symbols.indexOf(missingSymbol) === -1) {
                            this.missing_symbols.push(missingSymbol);
                            this.missing_symbol_count++;
                        }
                        this.missingSymbolsData[fileName].add(missingSymbol);
                    }
                }
            }
        }
    }
}
exports.CoronerdEvent = CoronerdEvent;
//# sourceMappingURL=coronerEvent.js.map