"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolGroup = void 0;
const path_1 = __importDefault(require("path"));
class SymbolGroup {
    constructor() {
        this.basenames = new Set();
        this.extensions = new Set();
        this.identifiers = new Set();
    }
    static fromMissing(isWindows, missing) {
        const groupData = {};
        for (const missingSymbol of missing) {
            const [fullFileName, debugId] = missingSymbol.split(',');
            let groupName = fullFileName;
            if (isWindows) {
                groupName = fullFileName.toLowerCase();
            }
            const extension = path_1.default.extname(groupName);
            const groupBase = path_1.default.basename(groupName, extension);
            const fileBaseName = path_1.default.basename(fullFileName, extension);
            let group = groupData[groupBase];
            if (!group) {
                group = new SymbolGroup();
                groupData[groupBase] = group;
            }
            group.basenames.add(fileBaseName);
            group.extensions.add(extension);
            group.identifiers.add(debugId);
        }
        if (isWindows) {
            for (const [_, group] of Object.entries(groupData)) {
                if (group.extensions.has('.dll') || group.extensions.has('.exe')) {
                    group.extensions.add('.pdb');
                }
                else if (group.extensions.has('.pdb')) {
                    group.extensions.add('.dll');
                    group.extensions.add('.exe');
                }
            }
        }
        return groupData;
    }
}
exports.SymbolGroup = SymbolGroup;
//# sourceMappingURL=symbolGroup.js.map