"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolService = void 0;
const common_1 = require("@nestjs/common");
const axios_1 = __importDefault(require("axios"));
const fs_extra_1 = require("fs-extra");
const https_1 = __importDefault(require("https"));
const path = __importStar(require("path"));
const symbolItemType_1 = require("../../database/entities/types/symbolItemType");
const env_1 = require("../../env");
const delay_1 = require("../../helpers/delay");
const binary_service_1 = require("../binary/binary.service");
const symbolType_1 = require("../binary/model/symbolType");
const config_service_1 = require("../config/config.service");
const coroner_service_1 = require("../coroner/coroner.service");
const directoryProvider_1 = require("../helpers/directoryProvider");
const processProvider_1 = require("../helpers/processProvider");
const logger_service_1 = require("../logger/logger.service");
const project_service_1 = require("../project/project.service");
const queue_service_1 = require("../queue/queue.service");
const symbol_item_cache_service_1 = require("../symbol-item/symbol-item-cache.service");
const symbol_item_service_1 = require("../symbol-item/symbol-item.service");
const symbol_server_logs_service_1 = require("../symbol-server-logs/symbol-server-logs.service");
const symbol_server_service_1 = require("../symbol-server/symbol-server.service");
const universe_service_1 = require("../universe/universe.service");
let SymbolService = class SymbolService {
    constructor(_config, _logger, _directoryProvider, _processProvider, _symbolServerService, _universeService, _projectService, _symbolServerLogsService, _binaryService, _coronerService, _symbolItemService, _symbolItemCacheService, _queueService) {
        var _a;
        this._config = _config;
        this._logger = _logger;
        this._directoryProvider = _directoryProvider;
        this._processProvider = _processProvider;
        this._symbolServerService = _symbolServerService;
        this._universeService = _universeService;
        this._projectService = _projectService;
        this._symbolServerLogsService = _symbolServerLogsService;
        this._binaryService = _binaryService;
        this._coronerService = _coronerService;
        this._symbolItemService = _symbolItemService;
        this._symbolItemCacheService = _symbolItemCacheService;
        this._queueService = _queueService;
        this.DefaultTimeout = 10 * 60 * 1000;
        this._httpsAgent = ((_a = env_1.service.configuration.descriptor) === null || _a === void 0 ? void 0 : _a.isOnPremise) ? new https_1.default.Agent({ rejectUnauthorized: false })
            : https_1.default.globalAgent;
    }
    async getStream(symbolName, debugId, universe, project = '') {
        const symbolServers = await this.getSymbolServersByName(universe, project, symbolName, debugId);
        if (symbolServers.length === 0) {
            throw new common_1.BadRequestException(`Cannot find any symbol server for ${symbolName}, ${debugId}. Symbol is in skip list?`);
        }
        return await this._binaryService.downloadStream(symbolName, debugId, symbolServers);
    }
    async decompress(event) {
        const symbols = {};
        for (const symbolName in event.missingSymbolsData) {
            const sym = event.missingSymbolsData[symbolName];
            this._logger.debug(`Considering ${symbolName} for decompression`);
            for (const missingSymbol of sym) {
                const debugId = missingSymbol.split(',')[1];
                if (!symbols[symbolName]) {
                    symbols[symbolName] = {};
                }
                else if (symbols[symbolName][debugId] !== undefined) {
                    if (symbols[symbolName][debugId] === true) {
                        this._logger.debug(`Skipping ${missingSymbol}: already satisfied`);
                        event.alreadyDownload = true;
                        delete event.missingSymbolsData[symbolName];
                    }
                    else {
                        this._logger.debug(`Skipping ${missingSymbol}: failed to satisfy`);
                    }
                    continue;
                }
                const symSubdir = path.join(symbolName, debugId);
                const candidate = await this.findSymbol(event, path.join(symSubdir, `${symbolName}.symb`));
                if (!candidate) {
                    this._logger.debug(`Skipping ${missingSymbol}: no symbol found`);
                    symbols[symbolName][debugId] = false;
                    continue;
                }
                this._logger.debug(`Trying ${missingSymbol} from ${candidate.symPath}: ` +
                    `found ${candidate.compressed ? '' : 'un'}compressed`);
                let satisfied = candidate.compressed === false;
                if (satisfied === false) {
                    satisfied = await this._processProvider.zstdDecompress(`${candidate.symPath}.zst`, candidate.symPath);
                    this._logger.debug(`${missingSymbol} in ${candidate.symPath}: ` +
                        `decompression ${satisfied ? 'succeeded' : 'failed'}`);
                }
                symbols[symbolName][debugId] = satisfied;
                if (satisfied === true) {
                    event.alreadyDownload = true;
                    delete event.missingSymbolsData[symbolName];
                }
            }
        }
    }
    async get(model, downloadPath, onlyCustomSymbolServers = false) {
        model.symbols = [];
        for (const missingSymbolGroupKey in model.missingSymbolsData) {
            const group = model.missingSymbolsData[missingSymbolGroupKey];
            this._logger.debug(`Symbol group ${missingSymbolGroupKey}: ${JSON.stringify(Array.from(group), null, 4)}`);
            const symbolFiles = [];
            for (const missingSymbol of group) {
                if (this._config.isProduction) {
                    const cacheData = this._symbolItemCacheService.symbolInCache(missingSymbolGroupKey, missingSymbol);
                    if (cacheData) {
                        this._logger.debug(`Symbol is already in cache. Skipping symbol ${missingSymbol}.`);
                        if (!cacheData.skipList) {
                            model.alreadyDownload = true;
                        }
                        continue;
                    }
                }
                let symbolServers = await this.getSymbolServers(model, missingSymbol);
                if (onlyCustomSymbolServers) {
                    symbolServers = symbolServers.filter((n) => n.universeId !== null || n.projectId !== null);
                }
                const result = await this._binaryService.download(downloadPath, model, missingSymbol, symbolServers);
                const emptyResult = !result || result.length === 0;
                this._symbolItemCacheService.addSymbolToCache(missingSymbolGroupKey, missingSymbol, emptyResult);
                if (emptyResult) {
                    this._logger.debug(`Cannot download symbols from any symbol server. Number of selected symbol servers ${symbolServers.length} for missing symbols: ${missingSymbol}`);
                    continue;
                }
                this._logger.debug(`Pushing symbols to coronerd output queue. Symbols: ${missingSymbol}`);
                symbolFiles.push(...result);
                await this._processProvider.extractCabinetsFromSymbols(result, downloadPath);
            }
            if (symbolFiles.length !== 0) {
                model.numberOfConvertedSymbols += await this.convertSymbols(downloadPath, symbolFiles);
                model.symbols.push(...symbolFiles);
            }
            await this._directoryProvider.remove(downloadPath);
        }
        await this._directoryProvider.remove(downloadPath);
        if (model.symbols.length === 0 && model.alreadyDownload !== true) {
            this._logger.info(`Cannot reprocess symbols because symbol servers didn't return any symbol. ObjectId: ${model.object_id}.`);
            return;
        }
        if (model.symbols.length === 0) {
            await delay_1.Delay.wait();
        }
        await this._coronerService.pushResult(model);
    }
    async getSymbolsStatus(universeName, projectName, symbolName, debugId, token) {
        const symbolServers = await this.getSymbolServersByName(universeName, projectName, symbolName, debugId);
        const ids = symbolServers.map((n) => n.id);
        const eventStack = this._queueService.getSymbolStatus(symbolName, debugId);
        return {
            numberOfSymbolServers: symbolServers.length,
            skipList: await this._symbolItemService.findByNameAndDebugIdInSymbolServers(symbolName, debugId, symbolItemType_1.SymbolItemType.SkipList, ids, true),
            whiteList: await this._symbolItemService.findByNameAndDebugIdInSymbolServers(symbolName, debugId, symbolItemType_1.SymbolItemType.WhiteList, ids, true),
            blackList: await this._symbolItemService.findByNameAndDebugIdInSymbolServers(symbolName, debugId, symbolItemType_1.SymbolItemType.BlackList, ids, true),
            logs: ids.length === 0
                ? []
                : await this._symbolServerLogsService.getByTextAndUniverseData(universeName, projectName, `symbol: ${symbolName} debugId: ${debugId}`),
            symbolsOnServer: await this.fetchManualSymbolInformation(symbolName, debugId, projectName, token),
            queueStatus: eventStack || undefined,
            downloadInProgress: this._queueService.isOnDownloadStage(symbolName, debugId),
        };
    }
    async fetchManualSymbolInformation(symbolName, debugId, projectName, token) {
        const requestData = {
            action: 'symbols',
            form: {},
        };
        try {
            const result = await axios_1.default.post(`${env_1.coronerdUrl}/api/symfile?project=${projectName}`, requestData, {
                httpsAgent: this._httpsAgent,
                headers: {
                    'X-Coroner-Token': token,
                    'X-Coroner-Location': env_1.coronerdUrl,
                },
            });
            if (result.status !== 200) {
                return false;
            }
            const data = result.data;
            if (!data || !data.response || !data.response.symbols || !data.response.symbols.files) {
                return false;
            }
            const tags = data.response.symbols;
            for (const rawTag of tags) {
                const tag = rawTag;
                const containElement = tag.files.some((n) => n.debug_file === symbolName && debugId === debugId);
                if (containElement) {
                    return true;
                }
            }
            return false;
        }
        catch (err) {
            return false;
        }
    }
    async convertSymbols(downloadPath, symbolFiles) {
        let numberOfConvertedSymbolds = 0;
        const downloadTask = symbolFiles[0].task;
        const elfSymbols = symbolFiles.filter((n) => n.type === symbolType_1.SymbolType.Elf);
        for (const elfSymbol of elfSymbols) {
            const dumpSymsResult = await this._processProvider.dumpSyms(elfSymbol.symbolPath, this._config.path.destination);
            await fs_extra_1.remove(elfSymbol.symbolPath);
            if (dumpSymsResult) {
                ++downloadTask.symbolServer.statistics.numberOfSuccessAtSymbolConversionStage;
                ++numberOfConvertedSymbolds;
            }
            else {
                ++downloadTask.symbolServer.statistics.numberOfFailuresAtSymbolConversionStage;
                if (this._config.isProduction) {
                    await this._symbolItemService.add({
                        symbolName: elfSymbol.symbolName,
                        debugId: elfSymbol.debugId,
                        symbolServerId: elfSymbol.task.symbolServer.id,
                        type: symbolItemType_1.SymbolItemType.SkipList,
                    });
                }
            }
        }
        if (elfSymbols.length === symbolFiles.length) {
            return numberOfConvertedSymbolds;
        }
        const conversionResult = await this._processProvider.pdbConvert(downloadPath, this._config.path.destination);
        if (!conversionResult) {
            if (this._config.isProduction) {
                for (const symbolFile of symbolFiles) {
                    await this._symbolItemService.add({
                        symbolName: symbolFile.symbolName,
                        debugId: symbolFile.debugId,
                        symbolServerId: symbolFile.task.symbolServer.id,
                        type: symbolItemType_1.SymbolItemType.SkipList,
                    });
                }
            }
            ++downloadTask.symbolServer.statistics.numberOfFailuresAtSymbolConversionStage;
            this._logger.debug(`Cannot convert files under directory: ${downloadPath}`);
        }
        else {
            numberOfConvertedSymbolds += symbolFiles.length;
            ++downloadTask.symbolServer.statistics.numberOfSuccessAtSymbolConversionStage;
        }
        return numberOfConvertedSymbolds;
    }
    async getSymbolServers(event, missingSymbols) {
        const [name, debugId] = missingSymbols.split(',');
        const projectId = event.universeEntity.project && event.universeEntity.project.length > 0
            ? event.universeEntity.project[0].id
            : undefined;
        return await this._symbolServerService.getAllByProjectName(event.universeEntity.id, projectId, name, debugId);
    }
    async getSymbolServersByName(universeName, projectName, symbolName, debugId) {
        const projectId = await this._projectService.getProjectIdByProjectName(projectName, universeName);
        const universeId = await this._universeService.getUniverseIdByUniverseName(universeName);
        if (!universeId) {
            throw new Error(`Universe ${universeName} doesn't exists`);
        }
        return await this._symbolServerService.getAllByProjectName(universeId, projectId, symbolName, debugId);
    }
    async findSymbol(event, symSubpath) {
        for (const root of this._config.coronerdDiskPaths) {
            const symRoot = path.join(root, event.universe, event.project, 'minidump');
            for (const tag of await this._directoryProvider.listAllFilesInDir(symRoot)) {
                const candidate = path.join(symRoot, tag);
                const symPath = path.join(candidate, symSubpath);
                let compressed = false;
                let ret = await this._directoryProvider.exists(symPath);
                if (ret === false) {
                    compressed = true;
                    ret = await this._directoryProvider.exists(`${symPath}.zst`);
                }
                if (ret === true) {
                    return { symPath, compressed };
                }
                this._logger.debug(`${symSubpath}: prefix ${candidate}: not found`);
            }
        }
        return undefined;
    }
};
SymbolService = __decorate([
    common_1.Injectable(),
    __param(12, common_1.Inject(common_1.forwardRef(() => queue_service_1.QueueService))),
    __metadata("design:paramtypes", [config_service_1.ConfigService,
        logger_service_1.LoggerService,
        directoryProvider_1.DirectoryProvider,
        processProvider_1.ProcessProvider,
        symbol_server_service_1.SymbolServerService,
        universe_service_1.UniverseService,
        project_service_1.ProjectService,
        symbol_server_logs_service_1.SymbolServerLogsService,
        binary_service_1.BinaryService,
        coroner_service_1.CoronerService,
        symbol_item_service_1.SymbolItemService,
        symbol_item_cache_service_1.SymbolItemCacheService,
        queue_service_1.QueueService])
], SymbolService);
exports.SymbolService = SymbolService;
//# sourceMappingURL=symbols.service.js.map