"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsStorage = void 0;
var axios_1 = __importDefault(require("axios"));
var os_1 = __importDefault(require("os"));
var logger_1 = require("../log/logger");
var MetricsStorage = /** @class */ (function () {
    function MetricsStorage(uuid, secret, opts, app, logger) {
        var _this = this;
        if (logger === void 0) { logger = logger_1.Logger.getLogger(); }
        this.uuid = uuid;
        this.secret = secret;
        this.opts = opts;
        this.logger = logger;
        this._hostUrl = 'https://api.circonus.com/module/httptrap/';
        this._url = "" + this._hostUrl + this.uuid + "/" + this.secret;
        this._defaultMetricsInterval = 60000;
        this._hostName = os_1.default.hostname().split('.')[0];
        this._data = {};
        if (!this.uuid || !this.secret) {
            if (logger) {
                logger.warn('Opts (or uuid/secret) object is required. Metrics are disabled');
            }
            return;
        }
        this.logger.debug('Initializing circonus metrics integrations');
        // override default circonus url.
        // different url might be useful in case
        // when we will prepare our own metrics client
        if (this.opts && this.opts.hostUrl) {
            this._hostUrl = this.opts.hostUrl;
        }
        if (app && (opts === null || opts === void 0 ? void 0 : opts.disableExpressMetrics) !== true) {
            app.use(function (req, res, next) { return _this.registerRequestLatency(req, res, next); });
        }
        this.addMetric('birthday', Date.now());
        setInterval(function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.beforeSend) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.beforeSend()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (Object.keys(this._data).length > 0) {
                            this.send();
                        }
                        return [2 /*return*/];
                }
            });
        }); }, (this.opts && this.opts.interval) || this._defaultMetricsInterval);
    }
    /**
     * Convert objects to metrics
     * @param source metrics object
     */
    MetricsStorage.prototype.add = function (source) {
        // metrics are disabled because of missing secret and uuid parameters
        if (!this.uuid || !this.secret) {
            return;
        }
        for (var key in source) {
            if (source.hasOwnProperty(key)) {
                var value = source[key];
                this.addMetric(key, value);
            }
        }
    };
    /**
     * Add single value to metrics
     * @param name metrics name
     * @param value metrics value
     */
    MetricsStorage.prototype.addMetric = function (name, value) {
        // metrics are disabled because of missing secret and uuid parameters
        if (!this.uuid || !this.secret) {
            return;
        }
        name = name + "|ST[hostname:" + this._hostName + "]";
        this._data[name] = this._data[name] || { _type: 'n', _value: [] };
        this._data[name]._value.push(value);
    };
    /**
     * Add single value to a histogram type metric
     * This is the same as addMetric except for _type: 'h'
     * @param name metrics name
     * @param value metrics value
     */
    MetricsStorage.prototype.addMetricHistogram = function (name, value) {
        // metrics are disabled because of missing secret and uuid parameters
        if (!this.uuid || !this.secret) {
            return;
        }
        name = name + "|ST[hostname:" + this._hostName + "]";
        this._data[name] = this._data[name] || { _type: 'h', _value: [] };
        this._data[name]._value.push(value);
    };
    MetricsStorage.prototype.registerRequestLatency = function (req, res, next) {
        var _this = this;
        if (!req.path) {
            return;
        }
        var startHrTime = process.hrtime();
        res.on('finish', function () {
            var elapsedHrTime = process.hrtime(startHrTime);
            var elapsedTimeInMs = elapsedHrTime[0] * 1000 + elapsedHrTime[1] / 1e6;
            _this.addMetric('express`' + req.method + '`' + req.path + '`latency', elapsedTimeInMs);
        });
        next();
    };
    MetricsStorage.prototype.send = function () {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            var data, error_1;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        data = this._data;
                        this._data = {};
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, axios_1.default.put(this._url, data)];
                    case 2:
                        _b.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _b.sent();
                        (_a = this.logger) === null || _a === void 0 ? void 0 : _a.warn(error_1, 'Cannot send metrics.');
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    return MetricsStorage;
}());
exports.MetricsStorage = MetricsStorage;
//# sourceMappingURL=metricsStorage.js.map