import express from 'express';
import { BacktraceService } from '../service/backtraceService';
import { Logger } from './../log/logger';
import { IErrorScopeOptions } from './errorScope';
export interface IAppOptions<T extends {}> {
    confPath?: string;
    remoteLogger?: boolean;
    logger?: Logger;
    /**
     * Express application instance. If app doesn't exists
     * BacktraceService will create default app instance.
     */
    app?: express.Express;
    /**
     * Mutation method, that allows developer to extend default instance of
     * ArgumentParser/Service object. Backtrace Service uses ArgumentParser object to
     * read service command line arguments and override service configuration.
     */
    serviceMutation?: (backtraceService: BacktraceService<T>) => void;
    coronerdCallback?: (ip: string) => void;
    /**
     * Options for the error scope middleware.
     */
    errorScopeOptions?: IErrorScopeOptions;
    /**
     * Package name - name of the final package. By default the package name will be set
     * to process.env.
     */
    packageName?: string;
}
