export declare type AttributeValue = string | number | boolean | object;
export declare class ErrorScope {
    private _attributes?;
    private readonly _attributeProviders;
    /**
     * Adds attributes to the attribute collection.
     * If there are already existing attributes, they will be replaced.
     * @param attributes Object containg attribute values.
     */
    addAttributes(attributes: Record<string, AttributeValue>): this;
    /**
     * Adds attributes to the attribute collection.
     * When the attributes are retrieved, the function will be run,
     * and the returned attributes will be added.
     * Use this method if you want to retrieve the attributes lazily.
     * If there are already existing attributes, they will be replaced.
     * @param attributesProvider Function returning attributes.
     */
    addAttributes(attributesProvider: () => Record<string, AttributeValue>): this;
    /**
     * Retrieves all attributes. If there were any functions added,
     * they will be run to retrieve attributes.
     * @returns Attributes added previously or retrieved from the methods.
     */
    getAttributes(): Record<string, AttributeValue> | undefined;
}
export interface IErrorScopeRequest {
    errorScope?: ErrorScope;
}
export interface IErrorScopeOptions {
    addExpressAttributes?: boolean;
}
