"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UniverseHelper = exports.validUniverseDomains = exports.http = exports.https = void 0;
var url = __importStar(require("url"));
exports.https = 'https://';
exports.http = 'http://';
// Any domains passed to a universe checking method
// should fail/return undefined if the domain isn't on this list
exports.validUniverseDomains = [
    '.backtrace.io',
    '.cloud-diagnostics.unity3d.com',
];
/**
 * Backtrae universe utils
 */
var UniverseHelper = /** @class */ (function () {
    function UniverseHelper() {
    }
    /**
     * return Backtrace host url
     * @param urlString sample url to any Backtrace resource
     * Example:
     * https://yolo.sp.backtrace.io => https://yolo.sp.backtrace.io
     * https://sampleHostName.backtrace.io/blah/poop/doop => https://sampleHostName.backtrace.io
     */
    UniverseHelper.getBacktraceHostName = function (urlString) {
        if (!urlString) {
            return undefined;
        }
        var obj = url.parse(urlString);
        return obj.protocol + "//" + obj.hostname;
    };
    /**
     * Get an universe name from Backtrace url.
     * @param urlString sample url to any Backtrace resource
     * Example:
     * https://yolo.sp.backtrace.io => yolo
     * https://sampleHostName.backtrace.io => sampleHostName
     */
    UniverseHelper.getBacktaceUniverseName = function (urlString) {
        if (!urlString) {
            return undefined;
        }
        var urlObj = new URL(urlString);
        if (urlObj.hostname === 'submit.backtrace.io') {
            // https://submit.backtrace.io/yolo/hash/json
            // pathname is /yolo/hash/json
            return urlObj.pathname.split('/', 2)[1];
        }
        // Verify if the domain is valid
        if (exports.validUniverseDomains.every(function (domain) { return !urlObj.hostname.endsWith(domain); })) {
            return undefined;
        }
        return urlObj.hostname.split('.', 1)[0];
    };
    UniverseHelper.getBacktaceUniverseNameFromConf = function (conf) {
        var universeName = conf.universe.name;
        if (!universeName || conf.universe.deleted !== 0) {
            throw new Error('cannot find universe');
        }
        return universeName;
    };
    UniverseHelper.removeProtocol = function (str) {
        if (str.startsWith(exports.https)) {
            return str.substring(exports.https.length);
        }
        if (str.startsWith(exports.http)) {
            return str.substring(exports.http.length);
        }
        return str;
    };
    // Abstraction in case it needs to be extended in future
    UniverseHelper.normalizeUrl = function (str) {
        return this.removeProtocol(str);
    };
    return UniverseHelper;
}());
exports.UniverseHelper = UniverseHelper;
//# sourceMappingURL=backtraceUniverseHelper.js.map